/*jslint node: true */
"use strict";

exports.default=
{
	AHTDPACLPlugin: function(api)
	{
	    return {
			ruleProcessingOrder:"deny,allow", // Order in which access control rules will be processed "deny,allow" or "allow,deny"
			exitOnRoleProcessingError:true, // Should we exit if an error occurs whilst processing the user role (boolean)
			sessionDataObjectNameOnConnection:"sessionData", // The sub-object of connection which hosts the user role property (string)
			roleStringNameOnSessionDataObject:"role", // The property of connection[sessionDataObjectNameOnConnection] which contains the user role (string)
			defaultRole:"public", // The role to apply to a user if none is found (in connection[sessionDataObjectNameOnConnection][roleStringNameOnSessionDataObject]). If null, false or empty, no role will be applied

			/* The access control rules. Format is:
			rules:
			{
				<ROLE1>:
				{
					inheritsFromRoles:["roletoInheritPermisionsFrom1","roletoInheritPermisionsFrom2"],
					allow:
					[
						"actionNameToAllow1",
						"actionNameToAllow2",
						"actionNameToAllow3"
					],
					deny:
					[
						"actionNameToDeny1",
						"actionNameToDeny2",
						"actionNameToDeny3"
					]
				},
				<ROLE2>
				...
			}
			*/

		// The below are a sample set of rules designed to make TDPAHCMS runnable out of the box - replace/amend them as you wish
			rules: 
			{
				admin:
				{
					inheritsFromRoles:["editor"],
					allow:
					[		
						"*"
					]
				},
				editor:
				{
					inheritsFromRoles:["public"],
					allow:
					[
						"ah-tdp-content-item-plugin/create",
						"ah-tdp-content-item-plugin/delete",
						"ah-tdp-content-item-plugin/getMany",
						"ah-tdp-content-item-plugin/publish",
						"ah-tdp-content-item-plugin/restore",
						"ah-tdp-content-item-plugin/unpublish",
						
					],
					deny:
					[
					]
				},
				public:
				{
					allow:
					[
						"ah-tdp-content-item-plugin/getByID",
						"ah-tdp-content-item-plugin/getNewest",

						"ah-tdp-auth-plugin/authenticateUser",
						"ah-tdp-auth-plugin/createUser",
						"ah-tdp-auth-plugin/deleteUser",
						"ah-tdp-auth-plugin/getUser",
						"ah-tdp-auth-plugin/updateUser",
						"ah-tdp-auth-plugin/verifyUserObject",
						"ah-tdp-auth-plugin/verifyUserProperty",

						"ah-tdp-session-plugin/loadSession",
						"ah-tdp-session-plugin/saveSession",


						// These two allows are just for the mocha tests for this (ACL) plugin
						"ah-tdp-acl-plugin/testActionOne",
						"ah-tdp-acl-plugin/testActionTwo",

						// Standard AH actions
						"cacheTest",
						"debug",
						"randomNumber",
						"showDocumentation",
						"sleepTest",
						"status"
					],
					deny:
					[
					]
				}
			}
		}
	}
};

exports.production=
{
	AHTDPACLPlugin: function(api)
	{
	    return {
			
	    }
	}
};

exports.uat=
{
	AHTDPACLPlugin: function(api)
	{
	    return {
			
	    }
	}
};

exports.development=
{
	AHTDPACLPlugin: function(api)
	{
	    return {
			
	    }
	}
};



